/* $Id: xiic_low_level_eeprom_example.c,v 1.1 2006/04/20 05:47:15 mta Exp $
*/
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002-2007 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/

/**************************************************************************
* Filename:     xiic_low_level_eeprom_example.c
* 
* Description:    
* This file consists of a polled mode design example which uses the Xilinx
* IIC device and low-level driver to exercise the EEPROM on a ML300/ML310
* board.
*
* The ML402 board has an IIC EEPROM on the board which is a 4Kbyte serial
* EEPROM(Microchip 24LC04B-I/ST data sheet).
*
* The WP pin of the EEPROM-Microchip 24LC32A has to be connected to ground 
* on the board to disable its hardware write protect
*
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a jhl  09/10/03 Created
* 1.00a sv   05/09/05 Minor changes to comply to Doxygen and coding guidelines
* 1.00a mta  03/09/06 Minor updates due to changes in the low level driver for
*                     supporting repeated start functionality.
* 2.00a pg   02/05/07 Updated to support the ML402 board 
* </pre>
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xparameters.h"
#include "xiic.h"
#include <stdio.h>

/************************** Constant Definitions *****************************/

/*
 * The following constants map to the XPAR parameters created in the
 * xparameters.h file. They are defined here such that a user can easily
 * change all the needed parameters in one place.
 */
#define IIC_BASE_ADDRESS    XPAR_IIC_EEPROM_BASEADDR

/*
 * The following constant defines the address of the IIC
 * device on the IIC bus. Note that since the address is only 7 bits, this
 * constant is the address divided by 2.
 */
#define EEPROM_ADDRESS 0x50     /* 0xA0 as an 8 bit number */

/*
 * The page size determines how much data should be written at a time.
 * The ML300 board supports a page size of 32 and 16
 * The write function should be called with this as a maximum byte count.
 */
#define PAGE_SIZE   16

/*
 * The Starting address in the IIC EEPROM on which this test is performed
 */
#define EEPROM_TEST_START_ADDRESS   128

#define printf xil_printf   /* A smaller footprint printf */

/*
 *  Add the define to print debug information
 *  #define PRINT_DEBUG_INFO
 *
 *  Add the define to print test information
 *  #define PRINT_TEST_INFO
 */
 
 #define PRINT_TEST_INFO

/**************************** Type Definitions *******************************/

/*
 * This value must be compatible with the hardware.
 * 
 * The ML402 supports 7 bit addressing
 *
 * Xuint16 for 10 bit addressing
 * Xuint8 for 7 bit addressing
 */
typedef Xuint8 AddressType;


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/

XStatus IicLowLevelEeprom();

XStatus ReadWriteVerify(AddressType Address);

unsigned EepromWriteByte(AddressType Address, Xuint8 *BufferPtr,
                         Xuint16 ByteCount);

unsigned EepromReadByte(AddressType Address, Xuint8 *BufferPtr,
                        Xuint16 ByteCount);

/************************** Variable Definitions **************************/

int ErrorCount;                       /* The Error Count */

Xuint8 WriteBuffer[PAGE_SIZE];        /* Write buffer for writing a page */

Xuint8 ReadBuffer[PAGE_SIZE];         /* Read buffer for reading a page */

Xuint8 ReadBufferAll[PAGE_SIZE * 4];  /* Buffer used for reading all the data */



/*****************************************************************************/
/**
* Main function to call the low level EEPROM example.
*
* @param    None
*
* @return   XST_SUCCESS if successful, XST_FAILURE if unsuccessful
*
* @note     None
*
******************************************************************************/
int main(void)
{
    XStatus Status;

    /*
     * Run the Low Level EEPROM example
     */
	 
	 #ifdef PRINT_TEST_INFO
		printf("\r\n\r\n\r\n\r\n** Testing EEPROM **\r\n\r\n");
	 #endif
    Status = IicLowLevelEeprom();
    if (Status != XST_SUCCESS)
    {
        return XST_FAILURE;
    }

    return XST_SUCCESS;
}

/*****************************************************************************/
/**
* The function uses the low level driver of IIC to read and write to the
* IIC EEPROM on the ML300 board. The addresses tested are from 128 to 192.
*
* @param    None
*
* @return   XST_SUCCESS if successful, XST_FAILURE if unsuccessful
*
* @note     None
*
****************************************************************************/
XStatus IicLowLevelEeprom()
{
    XStatus Status;
    unsigned BytesRead;

    /*
     * Read, write and verify a page of data at the address
     */
	  
	 #ifdef PRINT_TEST_INFO
		printf("\r\nIn IicLowLevelEeprom: \r\n");
		printf("Read, write, and verify a page of data at the address = 0x%x\r\n",EEPROM_TEST_START_ADDRESS );
    #endif
	 
	 Status = ReadWriteVerify(EEPROM_TEST_START_ADDRESS);
    if (Status != XST_SUCCESS)
    {
        ErrorCount++;
		  #ifdef PRINT_TEST_INFO
			 printf("Failure1\n\r");
		  #endif
    }

    /*
     * Read, write and verify a page of data at the address
     */
    #ifdef PRINT_TEST_INFO
		printf("\r\nIn IicLowLevelEeprom: \r\n");
		printf("Read, write, and verify a page of data at the address = 0x%x\n\r", (EEPROM_TEST_START_ADDRESS+PAGE_SIZE) );
	 #endif
	 
	 Status = ReadWriteVerify(EEPROM_TEST_START_ADDRESS + PAGE_SIZE);
    if (Status != XST_SUCCESS)
    {
        ErrorCount++;
		  #ifdef PRINT_TEST_INFO
			 printf("Failure2\n\r");
		  #endif
    }
	
    /*
     * Read, write and verify a page of data at the address
     */
    #ifdef PRINT_TEST_INFO
		printf("\r\nIn IicLowLevelEeprom: \r\n");
		printf("Read, write, and verify a page of data at the address = 0x%x\n\r", (EEPROM_TEST_START_ADDRESS+(PAGE_SIZE * 3)) );
	 #endif
	 
	 Status = ReadWriteVerify(EEPROM_TEST_START_ADDRESS + (PAGE_SIZE * 3));
    if (Status != XST_SUCCESS)
    {
        ErrorCount++;
		  #ifdef PRINT_TEST_INFO
			 printf("Failure3\n\r");
		  #endif
    }

    /*
     * Read, write and verify a page of data at the address
     */
    #ifdef PRINT_TEST_INFO
	   printf("\r\nIn IicLowLevelEeprom: \r\n");
		printf("Read, write, and verify a page of data at the address = 0x%x\n\r", (EEPROM_TEST_START_ADDRESS+(PAGE_SIZE * 2)) );
	 #endif
	 
	 Status = ReadWriteVerify(EEPROM_TEST_START_ADDRESS + (PAGE_SIZE * 2));
    if (Status != XST_SUCCESS)
    {
        ErrorCount++;
		  #ifdef PRINT_TEST_INFO
			 printf("Failure4\n\r");
		  #endif
    }

    /*
     * Read all the locations that were written in a single read,
     * this data is not verified, only read to show that a larger
     * amount of data can be read
     */
	 #ifdef PRINT_TEST_INFO
	   printf("\r\nIn IicLowLevelEeprom: \r\n");
		printf("Read all the locations that were written to.\r\n");
    #endif
	 
	 BytesRead = EepromReadByte(EEPROM_TEST_START_ADDRESS, ReadBufferAll,
                               PAGE_SIZE * 4);

    if (BytesRead != PAGE_SIZE * 4)
    {
        ErrorCount++;
		  #ifdef PRINT_TEST_INFO
			 printf("Failure5\n\r");
		  #endif
    }

    if (ErrorCount != 0x0)
    {
        Status = XST_FAILURE;
		  printf("\r\n*** A Test Failed ***\n\r");
    }
    else
    {
        Status = XST_SUCCESS;
		  printf("\r\n*** All Tests Completed Successfully ***\n\r");
    }

    return Status;
}

/*****************************************************************************/
/**
* This function writes, reads, and verifies the read to the IIC EEPROM.  It
* does the write as a single page write, performs a buffered read, and also
* performs byte reads.
*
* @param    Address is the starting address of the page in the EEPROM device
*           to which the data us written
*
* @return   XST_FAILURE if the test fails, or XST_SUCCESS if the test passed.
*
* @note     None
*
****************************************************************************/
XStatus ReadWriteVerify(AddressType Address)
{
    unsigned BytesWritten;
    unsigned BytesRead;
    int Index;

    /*
     * Initialize the data to written and the read buffer
     */
	  
	 #ifdef PRINT_DEBUG_INFO
		printf("In ReadWriteVerify: Initializing data write/read buffers.\r\n");
    #endif
	 for (Index = 0; Index < PAGE_SIZE; Index++)
    {
        WriteBuffer[Index] = Index;
        ReadBuffer[Index] = 0;
    }

    /*
     * Write to the EEPROM
     */
    BytesWritten = EepromWriteByte(Address, WriteBuffer, PAGE_SIZE);
    if (BytesWritten != PAGE_SIZE)
    {
        #ifdef PRINT_DEBUG_INFO
			 printf("In ReadWriteVerify: EepromWriteByte Failed\r\n");
		  #endif
		  return XST_FAILURE;
    }

    /*
     * Read from the EEPROM
     */
    BytesRead = EepromReadByte(Address, ReadBuffer, PAGE_SIZE);
    if (BytesRead != PAGE_SIZE)
    {
        #ifdef PRINT_DEBUG_INFO
			 printf("In ReadWriteVerify: EepromReadByte Failed\r\n");
		  #endif
		  return XST_FAILURE;
    }

    /*
     * Verify the data read against the data written
     */
    for (Index = 0; Index < PAGE_SIZE; Index++)
    {
        #ifdef PRINT_DEBUG_INFO
			 printf("In ReadWriteVerify: ReadBuffer = %d, WriteBuffer = %d\r\n",ReadBuffer[Index], WriteBuffer[Index]); 
		  #endif
		  
		  if (ReadBuffer[Index] != WriteBuffer[Index])
        {
            
				#ifdef PRINT_DEBUG_INFO
					printf("In ReadWriteVerify: ReadBuffer = %d, WriteBuffer = %d\r\n",ReadBuffer[Index], WriteBuffer[Index]); 
					printf("In ReadWriteVerify: Verify Failed\r\n");
				#endif
				return XST_FAILURE;
        }
        ReadBuffer[Index] = 0;
    }

    /*
     * Read each byte one at a time and verify
     */
    for (Index = 0; Index < PAGE_SIZE; Index++)
    {
        BytesRead = EepromReadByte(Address + Index,
                                   &ReadBuffer[Index], 1);
											  
		  #ifdef PRINT_DEBUG_INFO
			 printf("In ReadWriteVerify: ReadBuffer = %d\r\n",ReadBuffer[Index]); 
		  #endif
        if (BytesRead != 1)
        {
            return XST_FAILURE;
        }

        if (ReadBuffer[Index] != WriteBuffer[Index])
        {
            return XST_FAILURE;
        }
    }

    return XST_SUCCESS;
}

/*****************************************************************************/
/**
* This function writes a buffer of bytes to the IIC serial EEPROM.
*
* @param    Address contains the address in the EEPROM to write to.
* @param    BufferPtr contains the address of the data to write.
* @param    ByteCount contains the number of bytes in the buffer to be written.
*           Note that this should not exceed the page size of the EEPROM as
*           noted by the constant PAGE_SIZE.
*
* @return   The number of bytes written, a value less than that which was
*           specified as an input indicates an error.
*
* @note     None
*
****************************************************************************/
unsigned EepromWriteByte(AddressType Address,
                         Xuint8 *BufferPtr,
                         Xuint16 ByteCount)
{
    volatile unsigned SentByteCount;
    volatile unsigned AckByteCount;
    Xuint8 WriteBuffer[sizeof(Address) + PAGE_SIZE];
    int Index;

	 #ifdef PRINT_DEBUG_INFO
		printf("** In EepromWriteByte **\r\n\r\n");
    #endif
	 
	 /*
     * A temporary write buffer must be used which contains both the address
     * and the data to be written, put the address in first based upon the
     * size of the address for the EEPROM
     */
    if (sizeof(AddressType) == 2)
    {
        WriteBuffer[0] = (Xuint8)(Address >> 8);
        WriteBuffer[1] = (Xuint8)(Address);
    }
    else if (sizeof(AddressType) == 1)
    {
        WriteBuffer[0] = (Xuint8)(Address);
    }

    /*
     * Put the data in the write buffer following the address
     */
    for (Index = 0; Index < ByteCount; Index++)
    {
        WriteBuffer[sizeof(Address) + Index] = BufferPtr[Index];
    }

    /*
     * Set the address register to the specified address by writing
     * the address to the device, this must be tried until it succeeds
     * because a previous write to the device could be pending and it
     * will not ack until that write is complete
     */
    do
    {
        SentByteCount = XIic_Send(IIC_BASE_ADDRESS, EEPROM_ADDRESS,
                                  (Xuint8 *)&Address, sizeof(Address), 
                                  XIIC_STOP);
		  #ifdef PRINT_DEBUG_INFO
			 printf("In EepromWriteByte: Address = 0x%x, sizeof(Address) = %d\r\n", Address, sizeof(Address)); 
		  #endif
		  
	 } while (SentByteCount != sizeof(Address));

    /*
     * Write a page of data at the specified address to the EEPROM
     */
    SentByteCount = XIic_Send(IIC_BASE_ADDRESS, EEPROM_ADDRESS,
                              WriteBuffer, sizeof(Address) + PAGE_SIZE, 
                              XIIC_STOP);

    /*
     * Wait for the write to be complete by trying to do a write and
     * the device will not ack if the write is still active
     */
    do
    {
        AckByteCount = XIic_Send(IIC_BASE_ADDRESS, EEPROM_ADDRESS,
                                (Xuint8 *)&Address, sizeof(Address), XIIC_STOP);
    } while (AckByteCount != sizeof(Address));


    /*
     * Return the number of bytes written to the EEPROM
     */
	 #ifdef PRINT_DEBUG_INFO
		printf("In EepromWriteByte: Bytes Written = %d\r\n", (SentByteCount - sizeof(Address)));
		printf("** Leaving EepromWriteByte **\r\n\r\n");
    #endif
	 return SentByteCount - sizeof(Address);
}

/*****************************************************************************/
/**
* This function reads a number of bytes from the IIC serial EEPROM into a
* specified buffer.
*
* @param    Address contains the address in the EEPROM to read from.
* @param    BufferPtr contains the address of the data buffer to be filled.
* @param    ByteCount contains the number of bytes in the buffer to be read.
*           This value is not constrained by the page size of the device such
*           that up to 64K may be read in one call.
*
* @return   The number of bytes read.  A value less than the specified input
*           value indicates an error.
*
* @note     None
*
****************************************************************************/
unsigned EepromReadByte(AddressType Address,
                        Xuint8 *BufferPtr,
                        Xuint16 ByteCount)
{
    volatile unsigned ReceivedByteCount;
	 #ifdef PRINT_DEBUG_INFO
		printf("** In EepromReadByte **\r\n\r\n");
    #endif
	 
	 /*
     * Set the address register to the specified address by writing
     * the address to the device, this must be tried until it succeeds
     * because a previous write to the device could be pending and it
     * will not ack until that write is complete
     */
    do
    {
        ReceivedByteCount = XIic_Send(IIC_BASE_ADDRESS, EEPROM_ADDRESS,
                                      (Xuint8 *)&Address, sizeof(Address), 
                                      XIIC_STOP);
		  #ifdef PRINT_DEBUG_INFO
			 printf("In EepromReadByte: ReceivedByteCount = %d, %d\r\n", ReceivedByteCount, sizeof(Address));										  
		  #endif
	 } while (ReceivedByteCount != sizeof(Address));

    /*
     * Read the number of bytes at the specified address from the EEPROM
     */
    ReceivedByteCount = XIic_Recv(IIC_BASE_ADDRESS, EEPROM_ADDRESS,
                                  BufferPtr, ByteCount, XIIC_STOP);

    /*
     * Return the number of bytes read from the EEPROM
     */
    #ifdef PRINT_DEBUG_INFO
		printf ("In EepromReadByte: Read = %x \r\n\r\n", &BufferPtr);
		printf("** Leaving EepromReadByte **\r\n\r\n");
	 #endif
	 return ReceivedByteCount;
}



